﻿// Copyright (C) 2011 Oliver Sturm <oliver@oliversturm.com>. All rights reserved.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Net;
using System.Xml.Linq;
using System.Threading;

namespace QueryEbay.ManualAsync.Complete {
  /// <summary>
  /// Interaction logic for MainWindow.xaml
  /// </summary>
  public partial class MainWindow : Window {
    public MainWindow( ) {
      InitializeComponent( );
    }

    private void searchButton_Click(object sender, RoutedEventArgs e) {
      var searchString = searchEdit.Text;
      if (!String.IsNullOrEmpty(searchString)) {
        int maxitems;
        if (!Int32.TryParse(maxItemEdit.Text, out maxitems))
          maxitems = 20;
        itemView.Children.Clear( );
        GetItems(searchString, maxitems);
      }
    }

    private void cancelButton_Click(object sender, RoutedEventArgs e) {
      if (cancellationTokenSource != null)
        cancellationTokenSource.Cancel( );
    }

    CancellationTokenSource cancellationTokenSource;

    // Do your own thing? Read this:
    // http://ebayodata.cloudapp.net/docs
    const string searchUrlFormat = "http://ebayodata.cloudapp.net/Items?search={0}&$select=Title,Currency,CurrentPrice,ViewItemUrl,GalleryUrl&$top={1}&$skip={2}";
    //const string searchUrlFormat = "http://ebayodata.cloudapp.net/Items?search={0}";
    static readonly XNamespace ans = "http://www.w3.org/2005/Atom";
    static readonly XNamespace dns = "http://schemas.microsoft.com/ado/2007/08/dataservices";
    static readonly XNamespace mns = "http://schemas.microsoft.com/ado/2007/08/dataservices/metadata";

    void GetItems(string searchString, int pagesize, int skip,
      Action<IEnumerable<EbayItem>> processItems, CancellationToken cancelToken) {
      var url = String.Format(searchUrlFormat, searchString, pagesize, skip);
      var webclient = new WebClient( );
      cancelToken.Register(( ) => webclient.CancelAsync( ));

      webclient.DownloadStringCompleted += (sender, e) => {
        if (!e.Cancelled) {
          var entries =
            from entry in XDocument.Parse(e.Result).Descendants(ans + "entry")
            let propertiesNode = entry.Element(mns + "properties")
            select new EbayItem {
              Title = (string) entry.Element(ans + "title"),
              Currency = (string) propertiesNode.Element(dns + "Currency"),
              CurrentPrice = Convert.ToDecimal((string) propertiesNode.Element(dns + "CurrentPrice")),
              ImageUrl = (string) propertiesNode.Element(dns + "GalleryUrl"),
              ViewItemUrl = (string) propertiesNode.Element(dns + "ViewItemUrl")
            };
          processItems(entries);
        }
      };

      webclient.DownloadStringAsync(new Uri(url));
    }

    void GetItems(string searchString, int maxitems) {
      int skip = 0;
      int pagesize = Math.Min(2, maxitems);
      cancellationTokenSource = new CancellationTokenSource( );

      Action<IEnumerable<EbayItem>> processItems = null;
      processItems = items => {
        bool cont = true;
        if (items.Count( ) > 0)
          DisplayItems(items);
        else
          cont = false;
        skip += pagesize;
        if (cont && ((skip + pagesize) <= maxitems))
          GetItems(searchString, pagesize, skip, processItems, cancellationTokenSource.Token);
        else
          cancellationTokenSource = null;
      };

      GetItems(searchString, pagesize, skip, processItems, cancellationTokenSource.Token);
    }

    private void DisplayItems(IEnumerable<EbayItem> items) {
      foreach (var item in items) {
        var border = new Border( );
        border.BorderThickness = new Thickness(2);
        border.BorderBrush = Brushes.LightBlue;
        border.Margin = border.Padding = new Thickness(5);
        var panel = new StackPanel( );
        panel.Orientation = Orientation.Horizontal;
        if (!String.IsNullOrEmpty(item.ImageUrl))
          panel.Children.Add(new Image {
            Source = new BitmapImage(new Uri(item.ImageUrl)),
            Margin = new Thickness(6),
            MaxHeight = 150,
            MaxWidth = 150
          });
        var text = new TextBlock( );
        text.Inlines.Add(new Bold(new Run(item.Title)));
        text.Inlines.Add(new LineBreak( ));
        text.Inlines.Add(new Run(String.Format("{0} {1:#,##0.00}", item.Currency, item.CurrentPrice)));
        panel.Children.Add(text);
        border.Child = panel;
        itemView.Children.Add(border);
      }
    }

  }

  public class EbayItem {
    public string Title { get; set; }
    public string Currency { get; set; }
    public decimal CurrentPrice { get; set; }
    public string ImageUrl { get; set; }
    public string ViewItemUrl { get; set; }
  }
}
